<?php

namespace Drupal\universal_entity_api\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\universal_entity_api\Service\EntitySerializer;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for listing custom block entities via API.
 */
class BlockListController extends ControllerBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The entity serializer service.
   *
   * @var \Drupal\universal_entity_api\Service\EntitySerializer
   */
  protected EntitySerializer $serializer;

  /**
   * Constructs a BlockListController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $manager
   *   The entity type manager.
   * @param \Drupal\universal_entity_api\Service\EntitySerializer $serializer
   *   The entity serializer.
   */
  public function __construct(EntityTypeManagerInterface $manager, EntitySerializer $serializer) {
    $this->entityTypeManager = $manager;
    $this->serializer = $serializer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('universal_entity_api.serializer')
    );
  }

  /**
   * Returns a list of block_content entities for a given bundle.
   *
   * This method queries for block_content entities of a specific bundle,
   * loads them, normalizes them using the EntitySerializer, and returns
   * the results as a JSON response.
   *
   * @param string $bundle
   *   The block_content bundle machine name.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response containing the count and results.
   */
  public function list($bundle, Request $request) {
    $query = $this->entityTypeManager
      ->getStorage('block_content')
      ->getQuery()
      ->accessCheck(TRUE)
      ->condition('type', $bundle);

    $bids = $query->execute();
    $blocks = $this->entityTypeManager->getStorage('block_content')->loadMultiple($bids);
    $data = array_map(fn($block) => $this->serializer->normalize($block), $blocks);

    return new JsonResponse([
      'count' => count($data),
      'results' => $data,
    ]);
  }

}
