<?php

namespace Drupal\universal_entity_api\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\universal_entity_api\Service\EntitySerializer;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Controller for handling entity API requests.
 */
class EntityApiController extends ControllerBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The entity serializer service.
   *
   * @var \Drupal\universal_entity_api\Service\EntitySerializer
   */
  protected EntitySerializer $serializer;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected RequestStack $requestStack;

  /**
   * Constructs an EntityApiController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $manager
   *   The entity type manager.
   * @param \Drupal\universal_entity_api\Service\EntitySerializer $serializer
   *   The entity serializer.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(
    EntityTypeManagerInterface $manager,
    EntitySerializer $serializer,
    RequestStack $request_stack,
  ) {
    $this->entityTypeManager = $manager;
    $this->serializer = $serializer;
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('universal_entity_api.serializer'),
      $container->get('request_stack')
    );
  }

  /**
   * Handles entity API requests.
   *
   * This method loads an entity based on its type, bundle, and ID.
   * It then normalizes the entity data, optionally filtering by fields
   * provided in the request query, and returns a JSON response
   * with caching headers.
   *
   * @param string $entity_type
   *   The entity type machine name.
   * @param string $bundle
   *   The entity bundle machine name.
   * @param string $id
   *   The entity ID.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response containing the entity data or an error message.
   */
  public function handle($entity_type, $bundle, $id) {
    $entity = $this->entityTypeManager->getStorage($entity_type)->load($id);
    if (!$entity || $entity->bundle() !== $bundle) {
      return new JsonResponse(
        ['error' => 'Entity not found or bundle mismatch.'],
        404
      );
    }

    $request = $this->requestStack->getCurrentRequest();
    $fields = $request->query->get('fields');

    $data = $this->serializer->normalize($entity, ['fields' => $fields]);
    $response = new JsonResponse($data);
    $response->setMaxAge(3600);
    $response->headers->set('Cache-Control', 'public');

    return $response;
  }

}
