<?php

namespace Drupal\universal_entity_api\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Controller for listing entity types and their bundles via API.
 */
class EntityTypeListController extends ControllerBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs an EntityTypeListController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Returns a list of all entity types and their bundles.
   *
   * This method retrieves definitions for all entity types, and for each
   * entity type that has bundles, it loads and lists the bundle labels.
   * The information is returned as a JSON response.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response containing entity type and bundle information.
   */
  public function list() {
    $entity_types = $this->entityTypeManager->getDefinitions();
    $output = [];

    foreach ($entity_types as $type_id => $definition) {
      $bundles = [];
      if ($bundle_entity_type = $definition->getBundleEntityType()) {
        $bundle_storage = $this->entityTypeManager->getStorage($bundle_entity_type);
        $bundle_entities = $bundle_storage->loadMultiple();
        foreach ($bundle_entities as $bundle) {
          $bundles[] = $bundle->label();
        }
      }
      $output[$type_id] = [
        'label' => $definition->getLabel(),
        'bundles' => $bundles,
      ];
    }

    return new JsonResponse($output);
  }

}
