<?php

namespace Drupal\universal_entity_api\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\universal_entity_api\Normalizer\LayoutBuilderNormalizer;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Controller for retrieving layout builder data via API.
 */
class LayoutApiController extends ControllerBase {

  /**
   * The layout builder normalizer service.
   *
   * @var \Drupal\universal_entity_api\Normalizer\LayoutBuilderNormalizer
   */
  protected LayoutBuilderNormalizer $normalizer;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs a LayoutApiController object.
   *
   * @param \Drupal\universal_entity_api\Normalizer\LayoutBuilderNormalizer $normalizer
   *   The layout builder normalizer.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(LayoutBuilderNormalizer $normalizer, EntityTypeManagerInterface $entityTypeManager) {
    $this->normalizer = $normalizer;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('universal_entity_api.layout_builder_normalizer'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Returns the layout structure for a given entity.
   *
   * This method retrieves the layout builder structure for a specified
   * entity (identified by type and ID) and returns it as a JSON response.
   * It throws a NotFoundHttpException if the entity is not found.
   *
   * @param string $entity_type
   *   The entity type machine name.
   * @param string $entity_id
   *   The entity ID.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response containing the layout structure.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   If the entity is not found.
   */
  public function layout($entity_type, $entity_id) {
    $storage = $this->entityTypeManager->getStorage($entity_type);
    $entity = $storage->load($entity_id);

    if (!$entity) {
      throw new NotFoundHttpException('Entity not found');
    }

    $layout = $this->normalizer->getLayoutStructure($entity);

    return new JsonResponse([
      'entity_type' => $entity_type,
      'entity_id' => $entity_id,
      'layout' => $layout,
      'changed' => $entity->getChangedTime() ?? NULL,
    ]);
  }

}
