<?php

namespace Drupal\universal_entity_api\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Drupal\Core\Menu\MenuTreeParameters;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for retrieving menu tree data via API.
 */
class MenuController extends ControllerBase {

  /**
   * The menu link tree service.
   *
   * @var \Drupal\Core\Menu\MenuLinkTreeInterface
   */
  protected MenuLinkTreeInterface $menuTree;

  /**
   * Constructs a MenuController object.
   *
   * @param \Drupal\Core\Menu\MenuLinkTreeInterface $menuTree
   *   The menu link tree service.
   */
  public function __construct(MenuLinkTreeInterface $menuTree) {
    $this->menuTree = $menuTree;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('menu.link_tree')
    );
  }

  /**
   * Returns a menu tree for a given menu ID.
   *
   * This method retrieves the menu tree for the specified menu,
   * optionally limiting the depth of the tree based on a query parameter.
   * The menu tree is then transformed into a structured array and returned
   * as a JSON response.
   *
   * @param string $menu_id
   *   The machine name of the menu to retrieve.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response containing the menu tree data.
   */
  public function getMenu($menu_id, Request $request) {
    $depth = max(0, (int) $request->query->get('depth', 0));

    $params = new MenuTreeParameters();
    if ($depth > 0) {
      $params->setMaxDepth($depth);
    }

    $tree = $this->menuTree->load($menu_id, $params);

    $buildTree = function ($items) use (&$buildTree) {
      $output = [];
      foreach ($items as $item) {
        $link = $item->link;
        $url = $link->getUrlObject();
        $output[] = [
          'title' => $link->getTitle(),
          'url' => $url->toString(),
          'enabled' => $link->isEnabled(),
          'description' => $link->getDescription(),
          'weight' => $item->link->getWeight(),
          'external' => $url->isExternal(),
          'children' => $buildTree($item->subtree),
        ];
      }
      return $output;
    };

    return new JsonResponse($buildTree($tree));
  }

}
