<?php

namespace Drupal\universal_entity_api\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Controller for providing the OpenAPI specification.
 */
class OpenApiController extends ControllerBase {

  /**
   * Returns the OpenAPI specification for the Universal Entity API.
   *
   * This method defines the structure and endpoints of the API, including
   * details about available paths, parameters, and responses for various
   * entity types, menus, and configurations.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response containing the OpenAPI specification.
   */
  public function spec() {
    return new JsonResponse([
      'openapi' => '3.0.0',
      'info' => [
        'title' => 'Universal Entity API',
        'version' => '1.0.0',
        'description' => 'Dynamic content API for nodes, taxonomy, paragraphs, media, blocks, menus, and layouts. Supports sorting, field pruning, pagination, and OpenAPI integration.',
      ],
      'paths' => [
        '/api/{entity_type}/{bundle}/{id}' => [
          'get' => [
            'summary' => 'Fetch any entity with deeply resolved references',
            'parameters' => [
              [
                'name' => 'entity_type',
                'in' => 'path',
                'required' => TRUE,
                'schema' => ['type' => 'string'],
              ],
              [
                'name' => 'bundle',
                'in' => 'path',
                'required' => TRUE,
                'schema' => ['type' => 'string'],
              ],
              [
                'name' => 'id',
                'in' => 'path',
                'required' => TRUE,
                'schema' => ['type' => 'integer'],
              ],
              [
                'name' => 'fields',
                'in' => 'query',
                'schema' => ['type' => 'string'],
                'description' => 'Comma-separated list of fields to include',
              ],
            ],
            'responses' => ['200' => ['description' => 'Success']],
          ],
        ],
        '/api/node/{type}/list' => [
          'get' => [
            'summary' => 'List nodes with filtering, pagination, field pruning, and sorting',
            'parameters' => [
              [
                'name' => 'type',
                'in' => 'path',
                'required' => TRUE,
                'schema' => ['type' => 'string'],
              ],
              [
                'name' => 'limit',
                'in' => 'query',
                'schema' => ['type' => 'integer'],
              ],
              [
                'name' => 'page',
                'in' => 'query',
                'schema' => ['type' => 'integer'],
              ],
              [
                'name' => 'sort',
                'in' => 'query',
                'schema' => ['type' => 'string'],
                'description' => 'Field to sort by (prefix with - for DESC)',
              ],
              [
                'name' => 'fields',
                'in' => 'query',
                'schema' => ['type' => 'string'],
                'description' => 'Comma-separated list of fields to include',
              ],
            ],
            'responses' => ['200' => ['description' => 'Success']],
          ],
        ],
        '/api/block/{bundle}/list' => [
          'get' => [
            'summary' => 'List custom blocks by bundle',
            'parameters' => [
              [
                'name' => 'bundle',
                'in' => 'path',
                'required' => TRUE,
                'schema' => ['type' => 'string'],
              ],
              [
                'name' => 'fields',
                'in' => 'query',
                'schema' => ['type' => 'string'],
                'description' => 'Comma-separated list of fields to include',
              ],
              [
                'name' => 'limit',
                'in' => 'query',
                'schema' => ['type' => 'integer'],
              ],
              [
                'name' => 'page',
                'in' => 'query',
                'schema' => ['type' => 'integer'],
              ],
            ],
            'responses' => ['200' => ['description' => 'Success']],
          ],
        ],
        '/api/menu/{menu_id}' => [
          'get' => [
            'summary' => 'Fetch menu structure including nested links and associated fields',
            'parameters' => [
              [
                'name' => 'menu_id',
                'in' => 'path',
                'required' => TRUE,
                'schema' => ['type' => 'string'],
              ],
              [
                'name' => 'depth',
                'in' => 'query',
                'schema' => ['type' => 'integer'],
                'description' => 'How deep to go into nested menu links',
              ],
            ],
            'responses' => ['200' => ['description' => 'Success']],
          ],
        ],
        '/api/entity-type/list' => [
          'get' => [
            'summary' => 'List available entity types and their bundles',
            'responses' => ['200' => ['description' => 'Success']],
          ],
        ],
        '/api/config/basic' => [
          'get' => [
            'summary' => 'Fetch site configuration like site name, theme, logo',
            'responses' => ['200' => ['description' => 'Success']],
          ],
        ],
        '/api/layout/node/{id}' => [
          'get' => [
            'summary' => 'Get layout builder structure for a node including blocks and nested entities',
            'parameters' => [
              [
                'name' => 'id',
                'in' => 'path',
                'required' => TRUE,
                'schema' => ['type' => 'integer'],
              ],
            ],
            'responses' => ['200' => ['description' => 'Success']],
          ],
        ],
      ],
    ]);
  }

}
