<?php

namespace Drupal\universal_entity_api\Normalizer;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Normalizes field data for API output.
 */
class FieldNormalizer implements ContainerInjectionInterface {

  /**
   * The paragraph normalizer service.
   *
   * @var \Drupal\universal_entity_api\Normalizer\ParagraphNormalizer
   */
  protected ParagraphNormalizer $paragraphNormalizer;

  /**
   * The media normalizer service.
   *
   * @var \Drupal\universal_entity_api\Normalizer\MediaNormalizer
   */
  protected MediaNormalizer $mediaNormalizer;

  /**
   * The file URL generator service.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected FileUrlGeneratorInterface $fileUrlGenerator;

  /**
   * Constructs a FieldNormalizer object.
   *
   * @param \Drupal\universal_entity_api\Normalizer\ParagraphNormalizer $paragraphNormalizer
   *   The paragraph normalizer.
   * @param \Drupal\universal_entity_api\Normalizer\MediaNormalizer $mediaNormalizer
   *   The media normalizer.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $fileUrlGenerator
   *   The file URL generator.
   */
  public function __construct(
    ParagraphNormalizer $paragraphNormalizer,
    MediaNormalizer $mediaNormalizer,
    FileUrlGeneratorInterface $fileUrlGenerator,
  ) {
    $this->paragraphNormalizer = $paragraphNormalizer;
    $this->mediaNormalizer = $mediaNormalizer;
    $this->fileUrlGenerator = $fileUrlGenerator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('universal_entity_api.paragraph_normalizer'),
      $container->get('universal_entity_api.media_normalizer'),
      $container->get('file_url_generator')
    );
  }

  /**
   * Normalizes a field item list.
   *
   * This method processes different types of fields (entity references,
   * image fields, taxonomy terms) and normalizes their values into a
   * structured array for API output.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $field
   *   The field item list to normalize.
   *
   * @return array
   *   The normalized field data.
   */
  public function normalize(FieldItemListInterface $field) {
    $type = $field->getFieldDefinition()->getType();
    $entities = method_exists($field, 'referencedEntities') ? $field->referencedEntities() : [];

    // Paragraphs.
    if ($type === 'entity_reference_revisions' && !empty($entities)) {
      if ($entities[0]->getEntityTypeId() === 'paragraph') {
        return $this->paragraphNormalizer->normalize($entities);
      }
    }

    // Media.
    if ($type === 'entity_reference' && !empty($entities)) {
      if ($entities[0]->getEntityTypeId() === 'media') {
        return array_map(fn($media) => $this->mediaNormalizer->normalize($media), $entities);
      }

      // Fallback for other entity types — minimal data.
      return array_map(function (EntityInterface $e) {
        return [
          'id' => $e->id(),
          'type' => $e->getEntityTypeId(),
          'bundle' => $e->bundle(),
          'label' => $e->label(),
        ];
      }, $entities);
    }

    // Image fields.
    if ($type === 'image') {
      return array_map(function ($item) {
        if (!isset($item->entity)) {
          return [];
        }
        return [
          'url' => $this->fileUrlGenerator->generateAbsoluteString($item->entity->getFileUri()),
          'alt' => $item->alt ?? '',
          'title' => $item->title ?? '',
          'width' => $item->width ?? NULL,
          'height' => $item->height ?? NULL,
        ];
      }, iterator_to_array($field));
    }

    // Taxonomy term references.
    if (!empty($entities) && $entities[0]->getEntityTypeId() === 'taxonomy_term') {
      return array_map(function ($term) {
        return [
          'tid' => $term->id(),
          'name' => $term->label(),
          'description' => $term->getDescription(),
          'vocabulary' => $term->bundle(),
        ];
      }, $entities);
    }

    // Default.
    return $field->getValue();
  }

}
