<?php

namespace Drupal\universal_entity_api\Normalizer;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Path\PathAliasManagerInterface;
use Drupal\Core\Plugin\ContextAwarePluginInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\layout_builder\LayoutEntityHelperTrait;
use Drupal\layout_builder\Plugin\Block\InlineBlock;
use Drupal\Core\Block\BlockManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Normalizes Layout Builder data for API output.
 */
class LayoutBuilderNormalizer {

  use LayoutEntityHelperTrait {
    getSectionStorageForEntity as protected getSectionStorageForEntityTrait;
  }

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected RendererInterface $renderer;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected CacheBackendInterface $cache;

  /**
   * The file URL generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected FileUrlGeneratorInterface $fileUrlGenerator;

  /**
   * The path alias manager.
   *
   * @var \Drupal\Core\Path\PathAliasManagerInterface
   */
  protected PathAliasManagerInterface $pathAliasManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected TimeInterface $time;

  /**
   * The block plugin manager.
   *
   * @var \Drupal\Core\Block\BlockManagerInterface
   */
  protected BlockManagerInterface $blockManager;

  /**
   * Constructs a LayoutBuilderNormalizer object.
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache backend.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $fileUrlGenerator
   *   The file URL generator.
   * @param \Drupal\Core\Path\PathAliasManagerInterface $pathAliasManager
   *   The path alias manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Block\BlockManagerInterface $blockManager
   *   The block plugin manager.
   */
  public function __construct(
    RendererInterface $renderer,
    EntityTypeManagerInterface $entityTypeManager,
    CacheBackendInterface $cache,
    FileUrlGeneratorInterface $fileUrlGenerator,
    PathAliasManagerInterface $pathAliasManager,
    ModuleHandlerInterface $moduleHandler,
    TimeInterface $time,
    BlockManagerInterface $blockManager,
  ) {
    $this->renderer = $renderer;
    $this->entityTypeManager = $entityTypeManager;
    $this->cache = $cache;
    $this->fileUrlGenerator = $fileUrlGenerator;
    $this->pathAliasManager = $pathAliasManager;
    $this->moduleHandler = $moduleHandler;
    $this->time = $time;
    $this->blockManager = $blockManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('renderer'),
      $container->get('entity_type.manager'),
      $container->get('cache.default'),
      $container->get('file_url_generator'),
      $container->get('path_alias.manager'),
      $container->get('module_handler'),
      $container->get('datetime.time'),
      $container->get('plugin.manager.block')
    );
  }

  /**
   * Retrieves the layout structure for a given entity.
   *
   * This method extracts layout sections and their components from an entity
   * that uses Layout Builder. It attempts to render each component and
   * extracts relevant field data.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for which to retrieve the layout structure.
   *
   * @return array
   *   An array representing the layout structure, including
   *   sections and components.
   */
  public function getLayoutStructure(EntityInterface $entity): array {
    $layout_data = [];

    $section_storage = $this->getSectionStorageForEntityTrait($entity);
    if (!$section_storage) {
      return [];
    }

    foreach ($section_storage->getSections() as $section_index => $section) {
      $section_data = [
        'section_id' => $section_index,
        'layout_id' => $section->getLayoutId(),
        'components' => [],
      ];

      foreach ($section->getComponents() as $component) {
        $plugin = $component->getPlugin();
        // Attempt to recover if fields are empty.
        if (
          str_starts_with($plugin->getPluginId(), 'inline_block:') &&
          empty($plugin->getConfiguration()['block_id'])
        ) {
          $recovered_plugin = $this->attemptBlockRecovery($plugin);
          if ($recovered_plugin) {
            $plugin = $recovered_plugin;
          }
        }
        $fields = [];

        try {
          // Inject missing context for field_block and extra_field_block.
          // Inject context for field_block and extra_field_block plugins.
          if ($plugin instanceof ContextAwarePluginInterface) {
            $plugin_id = $plugin->getPluginId();

            if (str_starts_with($plugin_id, 'field_block:') ||
                str_starts_with($plugin_id, 'extra_field_block:')) {
              // 1. Inject 'entity' context (usually required).
              if ($plugin->getContextDefinition('entity')) {
                $plugin->setContextValue('entity', $entity);
              }

              // 2. Inject 'string' context (field name or extra field ID).
              if ($plugin->getContextDefinition('string')) {
                $parts = explode(':', $plugin_id);
                $field_name = end($parts);
                $plugin->setContextValue('string', $field_name);
              }
            }
          }

          // Build and render the plugin block.
          $build_array = $plugin->build();

          $this->renderer->renderRoot($build_array);

          $fields = $this->extractFieldsFromPlugin($plugin);
        }
        catch (\Exception $e) {
          $fields = ['error' => $e->getMessage()];
        }

        $section_data['components'][] = [
          'uuid' => $component->getUuid(),
          'region' => $component->getRegion(),
          'plugin_id' => $plugin->getPluginId(),
          'label' => method_exists($plugin, 'getTitle') ?
          $plugin->getTitle() :
          $plugin->getPluginId(),
          'fields' => $fields,
        ];
      }

      $layout_data[] = $section_data;
    }

    return $layout_data;
  }

  /**
   * Extracts fields from a given plugin.
   *
   * This method handles different types of plugins (inline blocks, standalone
   * block content, and generic context-aware plugins) to extract their
   * associated field data.
   *
   * @param object $plugin
   *   The plugin instance.
   *
   * @return array
   *   An array of extracted field data.
   */
  protected function extractFieldsFromPlugin($plugin): array {
    $fields = [];

    // Case 1: Inline Block - manually load via block_id.
    if ($plugin->getPluginId() === 'inline_block:' . $plugin->getDerivativeId()) {
      $config = $plugin->getConfiguration();
      if (!empty($config['block_id'])) {
        $block = $this->entityTypeManager->getStorage('block_content')->load($config['block_id']);
        if ($block && $block instanceof EntityInterface) {
          foreach ($block->getFields() as $field_name => $field) {
            if (!$field->isEmpty()) {
              $fields[$field_name] = $this->normalizeFieldValue($field);
            }
          }
          return $fields;
        }
      }
    }

    // Case 2: Standalone block_content.
    if (str_starts_with($plugin->getPluginId(), 'block_content:')) {
      $uuid = explode(':', $plugin->getPluginId())[1];
      // Use static cache to avoid repeated lookups for the same UUID.
      static $block_cache = [];
      if (!isset($block_cache[$uuid])) {
        // Try Drupal cache first.
        $cid = 'universal_entity_api:block_content:' . $uuid;
        $cache = $this->cache->get($cid);
        if ($cache && isset($cache->data) && $cache->expire > $this->time->getRequestTime()) {
          $block_data = $cache->data;
        }
        else {
          // Use entity query for UUID to get ID, then load by ID.
          $ids = $this->entityTypeManager->getStorage('block_content')->getQuery()
            ->accessCheck(FALSE)
            ->condition('uuid', $uuid)
            ->range(0, 1)
            ->execute();
          $block = !empty($ids)
            ? $this->entityTypeManager->getStorage('block_content')->load(reset($ids))
            : NULL;
          // Only cache serializable data.
          $block_data = NULL;
          if ($block) {
            $block_data = [];
            foreach ($block->getFields() as $field_name => $field) {
              if (!$field->isEmpty()) {
                $block_data[$field_name] = $this->normalizeFieldValue($field);
              }
            }
            // Cache for 1 hour.
            $this->cache->set($cid, $block_data, $this->time->getRequestTime() + 3600);
          }
        }
        $block_cache[$uuid] = $block_data;
      }
      $block_data = $block_cache[$uuid];
      if ($block_data) {
        foreach ($block_data as $field_name => $field_value) {
          $fields[$field_name] = $field_value;
        }
      }
    }

    // Case 4: Generic context-aware plugin extraction.
    if (method_exists($plugin, 'getContextMapping')) {
      foreach ($plugin->getContextMapping() as $context_id => $map) {
        try {
          if ($plugin->getContextDefinition($context_id) && $plugin->hasContext($context_id)) {
            $context = $plugin->getContext($context_id);
            if ($context->hasContextValue()) {
              $value = $context->getContextValue();

              if ($value instanceof FieldItemListInterface) {
                $fields[$value->getName()] = $this->normalizeFieldValue($value);
              }
              elseif ($value instanceof EntityInterface) {
                $fields[$context_id] = [
                  'entity_type' => $value->getEntityTypeId(),
                  'id' => $value->id(),
                  'label' => $value->label(),
                ];
              }
              else {
                $fields[$context_id] = $value;
              }
            }
          }
        }
        catch (\Exception $e) {
          $fields[$context_id] = ['error' => $e->getMessage()];
        }
      }
    }

    return $fields;
  }

  /**
   * Normalizes a field value.
   *
   * This method recursively normalizes field values, handling various field
   * types including entity references (paragraphs, media, taxonomy terms),
   * image fields, link fields, and text fields.
   *
   * @param mixed $field
   *   The field value to normalize. Can be a FieldItemListInterface or other.
   *
   * @return array
   *   The normalized field data.
   */
  protected function normalizeFieldValue($field): array {
    $output = [];

    if (!$field instanceof FieldItemListInterface) {
      return $output;
    }

    $field_type = $field->getFieldDefinition()->getType();
    $target_type = $field->getFieldDefinition()->getSetting('target_type');

    foreach ($field->getValue() as $item) {
      // Paragraph recursion.
      if ($field_type === 'entity_reference_revisions' && $target_type === 'paragraph') {
        foreach ($field->referencedEntities() as $paragraph) {
          $para_data = [
            'id' => $paragraph->id(),
            'type' => $paragraph->bundle(),
            'fields' => [],
          ];
          foreach ($paragraph->getFields() as $sub_field_name => $sub_field) {
            if (!$sub_field->isEmpty()) {
              $para_data['fields'][$sub_field_name] = $this->normalizeFieldValue($sub_field);
            }
          }
          $output[] = $para_data;
        }
      }

      // Media recursion (image, video, file, etc.)
      elseif ($field_type === 'entity_reference' && $target_type === 'media') {
        foreach ($field->referencedEntities() as $media) {
          $media_data = [
            'id' => $media->id(),
            'entity_type' => $media->getEntityTypeId(),
            'label' => $media->label(),
          ];

          // Extract file if exists (image, video, etc.)
          foreach ($media->getFields() as $media_field) {
            if ($media_field->getFieldDefinition()->getType() === 'image') {
              $image = $media_field->first();
              if ($image && isset($image->target_id)) {
                $file = $this->entityTypeManager->getStorage('file')->load($image->target_id);
                if ($file) {
                  $media_data['url'] = $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri());
                  $media_data['alt'] = $image->get('alt')->getString();
                  $media_data['title'] = $image->get('title')->getString();
                  $media_data['width'] = $image->get('width')->getString();
                  $media_data['height'] = $image->get('height')->getString();
                }
              }
            }
          }

          $output[] = $media_data;
        }
      }

      // Taxonomy term normalization.
      elseif ($field_type === 'entity_reference' && $target_type === 'taxonomy_term') {
        foreach ($field->referencedEntities() as $term) {
          $output[] = [
            'id' => $term->id(),
            'label' => $term->label(),
            'vid' => $term->bundle(),
            'description' => $term->getDescription(),
            'path' => $this->pathAliasManager->getAliasByPath('/taxonomy/term/' . $term->id()),
          ];
        }
      }

      // Direct image field (NOT media).
      elseif ($field_type === 'image' && isset($item['target_id'])) {
        $file = $this->entityTypeManager->getStorage('file')->load($item['target_id']);
        if ($file) {
          $image_data = [
            'target_id' => $item['target_id'],
            'url' => $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri()),
            'alt' => $item['alt'] ?? '',
            'title' => $item['title'] ?? '',
            'width' => $item['width'] ?? '',
            'height' => $item['height'] ?? '',
          ];
          $output[] = $image_data;
        }
      }

      // Link fields.
      elseif ($field_type === 'link') {
        $output[] = [
          'uri' => $item['uri'],
          'title' => $item['title'] ?? '',
          'options' => $item['options'] ?? [],
        ];
      }

      // Text fields.
      elseif (isset($item['value'])) {
        $output[] = [
          'value' => $item['value'],
          'format' => $item['format'] ?? NULL,
          'rendered_html' => $this->moduleHandler->invokeAll('check_markup', [
            $item['value'],
            $item['format'] ?? 'basic_html',
          ]),
          'rendered_text' => trim(strip_tags($item['value'])),
        ];
      }

      // Fallback: preserve raw.
      else {
        $output[] = $item;
      }
    }

    return $output;
  }

  /**
   * Attempts to recover an inline block if its block_id is missing.
   *
   * This method is a workaround for a potential issue where inline blocks
   * might lose their block_id but retain a block_revision_id. It loads the
   * revision, duplicates it to create a new block with a valid ID, and
   * returns a new InlineBlock plugin instance with the updated configuration.
   *
   * @param \Drupal\layout_builder\Plugin\Block\InlineBlock $plugin
   *   The inline block plugin instance.
   *
   * @return \Drupal\layout_builder\Plugin\Block\InlineBlock|null
   *   A new InlineBlock plugin instance if recovery is successful,
   *   or NULL otherwise.
   */
  protected function attemptBlockRecovery(InlineBlock $plugin): ?InlineBlock {
    $config = $plugin->getConfiguration();
    if (!empty($config['block_revision_id']) && empty($config['block_id'])) {
      $block_revision = $this->entityTypeManager
        ->getStorage('block_content')
        ->loadRevision($config['block_revision_id']);

      if ($block_revision) {
        // Clone and save a new block to get a clean ID.
        $new_block = $block_revision->createDuplicate();
        $new_block->isDefaultRevision(TRUE);
        $new_block->save();

        // Rebuild the plugin with updated config.
        $new_config = $config;
        $new_config['block_id'] = $new_block->id();
        $new_config['block_revision_id'] = $new_block->getRevisionId();

        /** @var \Drupal\layout_builder\Plugin\Block\InlineBlock $new_plugin */
        $new_plugin = $this->blockManager->createInstance(
          'inline_block:' . $new_block->bundle(),
          $new_config
        );

        return $new_plugin;
      }
    }

    return NULL;
  }

}
