<?php

namespace Drupal\universal_entity_api\Normalizer;

use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\media\MediaInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Normalizes media entities.
 */
class MediaNormalizer {

  /**
   * The file URL generator service.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected FileUrlGeneratorInterface $fileUrlGenerator;

  /**
   * Constructs a MediaNormalizer object.
   *
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $fileUrlGenerator
   *   The file URL generator.
   */
  public function __construct(FileUrlGeneratorInterface $fileUrlGenerator) {
    $this->fileUrlGenerator = $fileUrlGenerator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file_url_generator')
    );
  }

  /**
   * Normalizes a media entity.
   *
   * This method extracts relevant data from a media entity, specifically
   * focusing on image media, and returns it as a structured array.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media entity to normalize.
   *
   * @return array
   *   The normalized media data, or an empty array if the media is not an image
   *   or lacks necessary fields.
   */
  public function normalize(MediaInterface $media) {
    if (!$media->hasField('field_media_image')) {
      return [];
    }

    $image_field = $media->get('field_media_image');
    if ($image_field->isEmpty()) {
      return [];
    }

    $file = $image_field->entity;
    if (!$file) {
      return [];
    }

    return [
      'url' => $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri()),
      'alt' => $image_field->alt ?? '',
      'title' => $image_field->title ?? '',
      'mime' => $file->getMimeType(),
    ];
  }

}
