<?php

namespace Drupal\universal_entity_api\Normalizer;

use Drupal\Core\Menu\MenuLinkTreeInterface;
use Drupal\Core\Menu\MenuTreeParameters;

/**
 * Normalizes menu tree data.
 */
class MenuNormalizer {

  /**
   * The menu link tree service.
   *
   * @var \Drupal\Core\Menu\MenuLinkTreeInterface
   */
  protected MenuLinkTreeInterface $menuTree;

  /**
   * Constructs a MenuNormalizer object.
   *
   * @param \Drupal\Core\Menu\MenuLinkTreeInterface $menuTree
   *   The menu link tree service.
   */
  public function __construct(MenuLinkTreeInterface $menuTree) {
    $this->menuTree = $menuTree;
  }

  /**
   * Normalizes a menu tree for a given menu name.
   *
   * This method loads and builds the menu tree, then flattens it into a
   * simpler array structure suitable for API output.
   *
   * @param string $menu_name
   *   The machine name of the menu to normalize.
   *
   * @return array
   *   An array representing the normalized menu tree.
   */
  public function normalize(string $menu_name): array {
    $tree = $this->menuTree->load($menu_name, new MenuTreeParameters());
    $manipulated = $this->menuTree->build($tree);
    return $this->flatten($manipulated['#items']);
  }

  /**
   * Flattens a nested menu item array into a simpler structure.
   *
   * @param array $items
   *   An array of menu items, potentially nested.
   *
   * @return array
   *   A flattened array of menu items.
   */
  private function flatten(array $items): array {
    return array_map(function ($item) {
      return [
        'title' => $item['title'],
        'url' => $item['url']->toString(),
        'attributes' => $item['attributes']->toArray(),
        'children' => isset($item['below']) ? $this->flatten($item['below']) : [],
      ];
    }, $items);
  }

}
