<?php

namespace Drupal\universal_entity_api\Service;

use Drupal\Core\Entity\EntityInterface;
use Drupal\universal_entity_api\Normalizer\FieldNormalizer;

/**
 * Service for normalizing Drupal entities into a structured array.
 */
class EntitySerializer {

  /**
   * The field normalizer service.
   *
   * @var \Drupal\universal_entity_api\Normalizer\FieldNormalizer
   */
  protected $fieldNormalizer;

  /**
   * Constructs an EntitySerializer object.
   *
   * @param \Drupal\universal_entity_api\Normalizer\FieldNormalizer $fieldNormalizer
   *   The field normalizer.
   */
  public function __construct(FieldNormalizer $fieldNormalizer) {
    $this->fieldNormalizer = $fieldNormalizer;
  }

  /**
   * Normalizes a Drupal entity into a structured array.
   *
   * This method extracts an entity's ID, type, bundle, and label, and then
   * normalizes its fields using the FieldNormalizer. It supports filtering
   * fields based on a 'fields' parameter in the context.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to normalize.
   * @param array $context
   *   (optional) An associative array of context options.
   *   'fields': A comma-separated string or array of field names to include.
   *
   * @return array
   *   A structured array representing the normalized entity.
   */
  public function normalize(EntityInterface $entity, array $context = []) {
    $fields = $entity->getFields();
    $field_filter = [];

    // Support for both ?fields=title,body and ?fields[]=title&fields[]=body.
    if (!empty($context['fields'])) {
      if (is_string($context['fields'])) {
        $requested = array_map('trim', explode(',', $context['fields']));
      }
      elseif (is_array($context['fields'])) {
        $requested = array_map('trim', $context['fields']);
      }
      else {
        $requested = [];
      }
      $field_filter = array_flip($requested);
    }

    $data = [
      'id' => $entity->id(),
      'type' => [
        'id' => $entity->bundle(),
        'type' => $entity->getEntityTypeId() . '_type',
        'bundle' => $entity->getEntityTypeId() . '_type',
        'label' => $entity->getEntityType()->getLabel(),
      ],
      'bundle' => $entity->bundle(),
      'label' => $entity->label(),
    ];

    foreach ($fields as $field_name => $field) {
      if (!empty($field_filter) && !isset($field_filter[$field_name])) {
        // Skip fields not requested.
        continue;
      }
      $data[$field_name] = $this->fieldNormalizer->normalize($field);
    }

    return $data;
  }

}
