<?php

namespace Drupal\universal_entity_api\Service;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\layout_builder\LayoutEntityHelperTrait;
use Drupal\Core\Plugin\ContextAwarePluginInterface;

/**
 * Normalizes Layout Builder structures for entities.
 */
class LayoutBuilderNormalizer {
  use LayoutEntityHelperTrait;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected RendererInterface $renderer;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs a new LayoutBuilderNormalizer object.
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   */
  public function __construct(RendererInterface $renderer, EntityTypeManagerInterface $entityTypeManager) {
    $this->renderer = $renderer;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Retrieves the layout structure for a given entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to retrieve the layout structure for.
   *
   * @return array
   *   An array representing the layout structure.
   */
  public function getLayoutStructure(EntityInterface $entity): array {
    $layout_data = [];

    $section_storage = $this->getSectionStorageForEntity($entity);
    if (!$section_storage) {
      return [];
    }

    foreach ($section_storage->getSections() as $section_index => $section) {
      $section_data = [
        'section_id' => $section_index,
        'layout_id' => $section->getLayoutId(),
        'components' => [],
      ];

      foreach ($section->getComponents() as $component) {
        $plugin = $component->getPlugin();
        $rendered = '';
        $fields = [];

        try {
          // Inject missing context for field_block and extra_field_block.
          if (str_starts_with($plugin->getPluginId(), 'field_block:') || str_starts_with($plugin->getPluginId(), 'extra_field_block:')) {
            // Inject the current entity as 'entity' context.
            if (
              str_starts_with($plugin->getPluginId(), 'field_block:') ||
              str_starts_with($plugin->getPluginId(), 'extra_field_block:')
            ) {
              // Only if plugin supports contexts.
              if ($plugin instanceof ContextAwarePluginInterface) {
                // Set 'entity' context.
                if ($plugin->getContextDefinition('entity')) {
                  $plugin->setContextValue('entity', $entity);
                }

                // Set 'string' context (field name).
                if ($plugin->getContextDefinition('string')) {
                  $parts = explode(':', $plugin->getPluginId());
                  $field_name = end($parts);
                  $plugin->setContextValue('string', $field_name);
                }
              }
            }

            // Inject the field name as 'string' context.
            if ($plugin->getContextDefinition('string') && $plugin->hasContext('string')) {
              $parts = explode(':', $plugin->getPluginId());
              $field_name = end($parts);
              $plugin->setContextValue('string', $field_name);
            }
          }

          // Build and render the plugin block.
          $build_array = $plugin->build();
          $rendered = $this->renderer->renderRoot($build_array);
          $fields = $this->extractFieldsFromPlugin($plugin);
        }
        catch (\Exception $e) {
          $rendered = '<!-- render failed: ' . $e->getMessage() . ' -->';
          $fields = ['error' => $e->getMessage()];
        }

        $section_data['components'][] = [
          'uuid' => $component->getUuid(),
          'region' => $component->getRegion(),
          'plugin_id' => $plugin->getPluginId(),
          'label' => method_exists($plugin, 'getTitle') ? $plugin->getTitle() : $plugin->getPluginId(),
          'fields' => $fields,
          'rendered' => $rendered,
        ];
      }

      $layout_data[] = $section_data;
    }

    return $layout_data;
  }

  /**
   * Extracts fields from a given plugin.
   *
   * @param object $plugin
   *   The plugin object.
   *
   * @return array
   *   An array of extracted fields.
   */
  protected function extractFieldsFromPlugin($plugin): array {
    $fields = [];

    // Inline block field extraction.
    if (method_exists($plugin, 'getDerivativeId')) {
      $block_id = $plugin->getDerivativeId();
      if (str_starts_with($block_id, 'inline_block:')) {
        $uuid = str_replace('inline_block:', '', $block_id);
        $blocks = $this->entityTypeManager->getStorage('block_content')->loadByProperties(['uuid' => $uuid]);
        if ($block = reset($blocks)) {
          foreach ($block->getFields() as $field_name => $field) {
            if (!$field->isEmpty()) {
              $fields[$field_name] = $this->normalizeFieldValue($field);
            }
          }
        }
      }
    }

    // Context-aware plugins (field_block, views_block, etc).
    if (method_exists($plugin, 'getContextMapping')) {
      foreach ($plugin->getContextMapping() as $context_id => $map) {
        try {
          if ($plugin->getContextDefinition($context_id) && $plugin->hasContext($context_id)) {
            $context = $plugin->getContext($context_id);
            if ($context->hasContextValue()) {
              $value = $context->getContextValue();

              if ($value instanceof FieldItemListInterface) {
                $fields[$value->getName()] = $this->normalizeFieldValue($value);
              }
              elseif ($value instanceof EntityInterface) {
                $fields[$context_id] = [
                  'entity_type' => $value->getEntityTypeId(),
                  'id' => $value->id(),
                  'label' => $value->label(),
                ];
              }
              else {
                $fields[$context_id] = $value;
              }
            }
          }
        }
        catch (\Exception $e) {
          $fields[$context_id] = ['error' => $e->getMessage()];
        }
      }
    }

    return $fields;
  }

  /**
   * Normalizes a field value into a structured array.
   *
   * @param mixed $field
   *   The field item list or field value to normalize.
   *
   * @return array
   *   A normalized array representation of the field value.
   */
  protected function normalizeFieldValue($field): array {
    $output = [];

    if ($field instanceof FieldItemListInterface) {
      foreach ($field->getValue() as $item) {
        if ($field->getFieldDefinition()->getType() === 'entity_reference') {
          $target_type = $field->getFieldDefinition()->getSetting('target_type');
          $referenced = $field->referencedEntities();
          foreach ($referenced as $ref) {
            $out = [
              'id' => $ref->id(),
              'entity_type' => $ref->getEntityTypeId(),
              'label' => $ref->label(),
            ];
            if ($target_type === 'media' && $ref->bundle() === 'image' && $ref->hasField('field_media_image')) {
              $image = $ref->get('field_media_image')->first();
              if ($image) {
                $out['url'] = file_create_url($image->getValue()['target_id'] ? \Drupal::entityTypeManager()->getStorage('file')->load($image->getValue()['target_id'])->getFileUri() : '');
                $out['alt'] = $image->get('alt')->getString();
                $out['title'] = $image->get('title')->getString();
              }
            }
            $output[] = $out;
          }
        }
        elseif ($field->getFieldDefinition()->getType() === 'link') {
          $output[] = [
            'uri' => $item['uri'],
            'title' => $item['title'] ?? '',
            'options' => $item['options'] ?? [],
          ];
        }
        elseif (isset($item['value'])) {
          $output[] = [
            'value' => $item['value'],
            'format' => $item['format'] ?? NULL,
            'rendered_html' => check_markup($item['value'], $item['format'] ?? 'basic_html'),
            'rendered_text' => trim(strip_tags($item['value'])),
          ];
        }
        else {
          $output[] = $item;
        }
      }
    }

    return $output;
  }

}
